# -*-: coding utf-8 -*-

import os

from ..base import Base
from ...utils.pip_installer import PipInstaller
from ...utils.os_helpers import file_exists
from ...utils.snipsfile import Snipsfile
from .skill import SkillInstaller, SkillInstallerWarning

from ... import DEFAULT_SNIPSFILE_PATH

from snipsskillscore import pretty_printer as pp

class SkillsInstallerException(Exception):
    pass

class SkillsInstallerWarning(Exception):
    pass


class SkillsInstaller(Base):
    
    def run(self):
        try:
            SkillsInstaller.install(self.options['--snipsfile'], self.options['--force-download'])
        except SkillsInstallerWarning as e:
            pp.pwarning(str(e))
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def install(snipsfile_path=None, silent=False, force_download=False):
        SkillsInstaller.print_start(silent)
        if snipsfile_path is None:
            snipsfile_path = DEFAULT_SNIPSFILE_PATH
        if snipsfile_path is not None and not file_exists(snipsfile_path):
            raise SkillsInstallerException("Error installing skills: Snipsfile not found")
        snipsfile = Snipsfile(snipsfile_path)
        num_installed = SkillsInstaller.install_from_snipsfile(snipsfile, silent=True, force_download=force_download)
        SkillsInstaller.print_done(num_installed, silent)


    @staticmethod
    def install_from_snipsfile(snipsfile, silent=False, force_download=False):
        SkillsInstaller.print_start(silent)
        if snipsfile is None:
            raise SkillsInstallerException("Error installing skills: no Snipsfile provided")
        skill_urls = snipsfile.get_skill_urls()
        if len(skill_urls) == 0:
            raise SkillsInstallerWarning("No skills found in Snipsfile. Skipping skill installation")
        num_installed = SkillsInstaller.install_from_urls(skill_urls, silent=True, force_download=force_download)
        SkillsInstaller.print_done(num_installed, silent)
        return num_installed


    @staticmethod
    def install_from_urls(skill_urls, silent=False, force_download=False):
        SkillsInstaller.print_start(silent)
        num_installed = 0
        for url in skill_urls:
            try:
                SkillInstaller.install(url, force_download=force_download)
                num_installed = num_installed + 1
            except SkillInstallerWarning as e:
                pp.pwarning(str(e))
            except Exception as e:
                pp.perror(str(e))
        SkillsInstaller.print_done(num_installed, silent)
        return num_installed


    @staticmethod
    def print_start(silent=False):
        if not silent:
            pp.pcommand("Installing skills")


    @staticmethod
    def print_done(num_installed, silent=False):
        if not silent:
            pp.psuccess("Successfully installed {} skills".format(num_installed))



# -*-: coding utf-8 -*-

import os
import shutil
import time

from ..base import Base
from ..session.login import Login
from ...utils.http_helpers import fetch_url
from ...utils.os_helpers import write_binary_file, file_exists, is_raspi_os
from ...utils.cache import Cache
from ...utils.snips import Snips
from ...utils.intent_class_generator import IntentClassGenerator
from .fetch import AssistantFetcher

from ... import SNIPS_CACHE_INTENTS_DIR

from snipsskillscore import pretty_printer as pp


class AssistantLoaderException(Exception):
    pass


# pylint: disable=too-few-public-methods
class AssistantLoader(Base):

    def run(self):
        """ Command runner.

        Docopt command:
        
        snipsskills load assistant [--file=<file> --platform-only]
        """
        try:
            generate_classes = not self.options['--platform-only']
            AssistantLoader.load(self.options['--file'], generate_classes=generate_classes)
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def load(file_path=None, generate_classes=True):
        pp.pcommand("Loading assistant")

        if file_path is None:
            file_path = AssistantFetcher.SNIPS_TEMP_ASSISTANT_PATH

        message = pp.ConsoleMessage("Loading assistant from file {}".format(file_path))
        message.start()

        if file_path is not None and not file_exists(file_path):
            message.error()
            raise AssistantLoaderException("Error loading assistant: file {} not found".format(file_path))

        if is_raspi_os():
            if not Snips.is_installed():
                message.error()
                raise AssistantLoaderException("Error: loading an assistant requires the Snips platform to be installed. Please run 'curl https://install.snips.ai -sSf | sh' to install the Snips Platform")
            Snips.load_assistant(file_path)

        message.done()

        if generate_classes:
            AssistantLoader.generate_intent_classes(file_path)

        pp.psuccess("Assistant has been successfully loaded")

    @staticmethod
    def generate_intent_classes(file_path):
        message = pp.ConsoleMessage("Generating classes from assistant model")
        message.start()
        try:
            shutil.rmtree(SNIPS_CACHE_INTENTS_DIR)
        except Exception:
            pass
        IntentClassGenerator().generate(file_path, SNIPS_CACHE_INTENTS_DIR)
        message.done()



# -*-: coding utf-8 -*-

from ..base import Base
from .logout import Logout
from snipsskillscore import pretty_printer as pp
from ...utils.os_helpers import write_text_file, read_file, file_exists, ask_for_input, ask_for_password
from ...utils.cache import Cache
from ...utils.auth import Auth


class InvalidTokenException(Exception):
    pass


class Login(Base):

    def run(self):
        try:
            Login.login(email=self.options['--email'], password=self.options['--password'])
        except Exception as e:
            pp.perror("Error logging in: {}".format(str(e)))


    @staticmethod
    def login(email=None, password=None, greeting=None, silent=False):
        has_credentials = email is not None and password is not None
        silent = silent or has_credentials
        if has_credentials:
            Logout.logout()
        token = Cache.get_login_token()
        if not token:
            if not has_credentials:
                pp.pcommand(greeting or "Please enter your Snips Console credentials")
                email = ask_for_input("Email address:")
                password = ask_for_password("Password:")
            token = Auth.retrieve_token(email, password)
            if token is not None:
                Cache.save_login_token(token)
                if not silent:
                    pp.psuccess("You are now signed in")
            else:
                raise InvalidTokenException("Could not validate authentication token")
        else:
            if not silent:
                pp.psuccess("You are already signed in")
        return token



# -*-: coding utf-8 -*-
""" Downloader for Snips assistants. """

from http_helpers import post_request_json

class Auth:

    AUTH_URL = "https://external-gateway.snips.ai/v1/user/auth"

    @staticmethod
    def retrieve_token(email, password):
        data = { 'email': email, 'password': password }
        response, response_headers = post_request_json(Auth.AUTH_URL, data)
        token = response_headers.getheader('Authorization')
        return token



# -*-: coding utf-8 -*-

import os

from ..base import Base
from ...utils.os_helpers import is_raspi_os, is_node_available, execute_command, download_file, file_exists

from ... import prepare_cache, NODE_MODULES_LOCATION

from snipsskillscore import pretty_printer as pp

class BluetoothInstallerException(Exception):
    pass


class BluetoothInstaller(Base):
    
    SNIPS_MQTT_RELAY_MODULE_NAME = "snips-mqtt-relay"

    def run(self):
        try:
            BluetoothInstaller.install(force_download=self.options['--force-download'])
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def install(force_download=False):
        pp.pcommand("Setting up Bluetooth")

        if not is_raspi_os():
            raise BluetoothInstallerException("Error: Bluetooth is only available on Raspberry Pi")

        if not is_node_available():
            BluetoothInstaller.install_node()
        
        if force_download and not BluetoothInstaller.is_snips_mqtt_relay_installed():
            message = pp.ConsoleMessage("Installing Node module $GREEN{}$RESET".format(BluetoothInstaller.SNIPS_MQTT_RELAY_MODULE_NAME))
            message.start()
            try:
                execute_command("sudo npm install --no-cache --prefix={} {}".format(NODE_MODULES_LOCATION, BluetoothInstaller.SNIPS_MQTT_RELAY_MODULE_NAME), True)
                message.done()
            except:
                message.error()
                raise BluetoothInstallerException("Error: Error installing Bluetooth module {}. Please install it manually".format(BluetoothInstaller.SNIPS_MQTT_RELAY_MODULE_NAME))

        pp.psuccess("Bluetooth is successfully installed")


    @staticmethod
    def is_snips_mqtt_relay_installed():
        return os.path.isdir(os.path.join(NODE_MODULES_LOCATION, BluetoothInstaller.SNIPS_MQTT_RELAY_MODULE_NAME))
        

    @staticmethod
    def install_node():
        pp.psubmessage("Node is not available. Installing node.", indent=True)
        message = pp.ConsoleMessage("Removing previous version of Node")
        message.start()
        try:
            execute_command("sudo apt-get -y remove nodejs nodejs-legacy npm")
            message.done()
        except:
            message.error()
            pass


        deb_file = "node_latest_armhf.deb"
        deb_url = "http://node-arm.herokuapp.com/node_latest_armhf.deb"
        
        message = pp.ConsoleMessage("Downloading Raspbian-compatible version of Node")
        message.start()

        try:
            download_file(deb_url, deb_file)
            message.done()
        except:
            message.error()
            raise BluetoothInstallerException("Error: failed to download compatible version of Node")


        message = pp.ConsoleMessage("Installing Node")
        message.start()
        try:
            execute_command("sudo dpkg -i {}".format(deb_file))
            message.done()
        except:
            message.error()
            raise BluetoothInstallerException("Error: installing Node. Please install Node manually")

        try:
            remove_file(filename)
        except:
            pass

        pp.psuccess("Successfully installed Node")



# -*-: coding utf-8 -*-
""" snipsskills module """
__version__ = '0.1.6.21'

import os
import logging

HOME_DIR = os.path.expanduser('~')
if 'arm' in " ".join(os.uname()):
    HOME_DIR = "/home/pi"

SNIPS_CACHE_DIR_NAME = ".snips"
SNIPS_CACHE_DIR = os.path.join(HOME_DIR, SNIPS_CACHE_DIR_NAME)
NODE_MODULES_LOCATION = SNIPS_CACHE_DIR
NODE_MODULES_PATH = os.path.join(NODE_MODULES_LOCATION, "node_modules")
DEFAULT_SNIPSFILE_PATH = os.path.join(os.getcwd(), "Snipsfile")
SNIPS_CACHE_INTENTS_DIR = os.path.join(SNIPS_CACHE_DIR, "intents")
SNIPS_CACHE_INTENT_REGISTRY_FILE = os.path.join(SNIPS_CACHE_INTENTS_DIR, "intent_registry.py")

DEB_VENV = "/opt/venvs/snipsskills"
SHELL_COMMAND = "/bin/bash"
PIP_BINARY = os.path.join(DEB_VENV, "bin/pip")

ASOUNDRC_DEST_PATH = os.path.join(HOME_DIR, ".asoundrc")
ASOUNDCONF_DEST_PATH = "/etc/asound.conf"

def prepare_cache():
    if not os.path.exists(HOME_DIR):
        os.makedirs(HOME_DIR)
    if not os.path.exists(SNIPS_CACHE_DIR):
        os.makedirs(SNIPS_CACHE_DIR)

prepare_cache()

logger = logging.getLogger(__name__)
handler = logging.StreamHandler()
log_format = '\033[2m%(asctime)s\033[0m [%(levelname)s] %(message)s'
date_format = '%Y-%m-%d %H:%M:%S'
formatter = logging.Formatter(log_format, date_format)
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)

RESET_SEQ = u'\033[0m'
GREEN_COLOR = u'\033[32m'
BLUE_COLOR = u'\033[34m'



# -*-: coding utf-8 -*-
"""The microphone setup command."""

import time

from ...base import Base
from ....utils.os_helpers import is_raspi_os, is_node_available, which, file_exists
from ....utils.systemd import Systemd
from ....utils.snipsfile import Snipsfile

from .... import NODE_MODULES_PATH, DEFAULT_SNIPSFILE_PATH

from snipsskillscore import pretty_printer as pp

class SystemdBluetoothException(Exception):
    pass


class SystemdBluetooth(Base):

    SNIPSBLE_SERVICE_NAME = "snipsble"
    SNIPSBLE_SERVICE_UUID = "13EA4259-9D9E-42D1-A78B-638ED22CC768"
    SNIPSBLE_CHARACTERISTIC_UUID = "81D97A06-7A2D-4A98-A2E2-41688E3D8283"
    SNIPSBLE_MODULE_NAME = "snips-mqtt-relay"
    SNIPSBLE_SCRIPT = "{node_bin_path} {node_module_path}/{module_name}/index.js --serviceUUID={serviceUUID} --characteristicUUID={characteristicUUID} {mqtt_hostname} {mqtt_port}"

    def run(self):
        try:
            mqtt_hostname = self.options['--mqtt-host']
            mqtt_port = self.options['--mqtt-port']
            if mqtt_hostname is not None or mqtt_port is not None:
                SystemdBluetooth.setup_from_params(mqtt_hostname=mqtt_hostname, mqtt_port=mqtt_port)
            else:
                SystemdBluetooth.setup(self.options['--snipsfile'])
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def setup(snipsfile_path=None):
        if snipsfile_path is None:
            snipsfile_path = DEFAULT_SNIPSFILE_PATH
        if snipsfile_path is not None and not file_exists(snipsfile_path):
            raise SystemdBluetoothException("Error setting up Bluetooth systemd: Snipsfile not found")
        snipsfile = Snipsfile(snipsfile_path)
        SystemdBluetooth.setup_from_snipsfile(snipsfile)


    @staticmethod
    def setup_from_snipsfile(snipsfile):
        if snipsfile is None:
            raise SystemdBluetoothException("Error setting up Bluetooth systemd: Snipsfile not found")
        SystemdBluetooth.setup_from_params(snipsfile.mqtt_hostname, snipsfile.mqtt_port)


    @staticmethod
    def setup_from_params(mqtt_hostname="localhost", mqtt_port=9898):
        pp.pcommand("Setting up Bluetooth as a Systemd service")

        if not is_raspi_os():
            raise SystemdBluetoothException("Bluetooth Systemd configuration is only available on Raspberry Pi. Skipping Systemd setup")

        if not is_node_available():
            raise SystemdBluetoothException("Error: Bluetooth module must be installed. Run 'snipsskills install bluetooth' to setup Bluetooth")

        contents = Systemd.get_template(SystemdBluetooth.SNIPSBLE_SERVICE_NAME)
        if contents is None:
            return

        node_bin_path = which('node')            
        command = SystemdBluetooth.SNIPSBLE_SCRIPT.format(
                node_bin_path=node_bin_path,
                node_module_path=NODE_MODULES_PATH,
                module_name=SystemdBluetooth.SNIPSBLE_MODULE_NAME,
                serviceUUID=SystemdBluetooth.SNIPSBLE_SERVICE_UUID,
                characteristicUUID=SystemdBluetooth.SNIPSBLE_CHARACTERISTIC_UUID,
                mqtt_hostname=mqtt_hostname,
                mqtt_port=mqtt_port
            )

        contents = contents.replace("{{SNIPSBLE_COMMAND}}", command)
        Systemd.write_systemd_file(SystemdBluetooth.SNIPSBLE_SERVICE_NAME, None, contents)
        Systemd.enable_service(None, SystemdBluetooth.SNIPSBLE_SERVICE_NAME)

        pp.psuccess("Successfully set up Bluetooth as a Systemd service")



# -*-: coding utf-8 -*-
""" Utilities for managing the Snips SDK. """

import os
import subprocess

from .os_helpers import cmd_exists, is_raspi_os, ask_yes_no

# from snipsskillscore.logging import log, log_success, log_warning, log_error

try:
    from subprocess import DEVNULL
except ImportError:
    DEVNULL = open(os.devnull, 'wb')

# pylint: disable=too-few-public-methods


SNIPS_INSTALL_COMMAND = "curl https://install.snips.ai -sSf"
SNIPS_INSTALL_ASSISTANT_COMMAND = "snips-install-assistant {}"


class SnipsUnsupportedPlatform(Exception):
    """ Unsupported platform exception class. """
    pass


class SnipsNotFound(Exception):
    """ Snips command not found exception class. """
    pass


class SnipsRuntimeFailure(Exception):
    """ Snips runtime failure exception class. """
    pass


class SnipsInstallationFailure(Exception):
    """ Snips installation failure exception class. """
    pass


class Snips:
    """ Utilities for managing the Snips SDK. """

    # @staticmethod
    # def install(answer_yes=None):
    #     """ Install the Snips SDK. """
    #     if Snips.is_installed():
    #         return

    #     if ask_yes_no("Would you like to install the Snips SDK?", answer_yes) == False:
    #         return

    #     log("Installing the Snips SDK")

    #     if not is_raspi_os():
    #         raise SnipsUnsupportedPlatform()

    #     curl_command = subprocess.Popen(
    #         SNIPS_INSTALL_COMMAND.split(), stdout=subprocess.PIPE)
    #     sh_command = subprocess.Popen("sh", stdin=curl_command.stdout)
    #     output, error = sh_command.communicate()
    #     rc = sh_command.returncode

    #     if (rc > 0):
    #         raise SnipsInstallationFailure(output)
    #     else:
    #         log_success("The Snips SDK was successfully installed")

    # @staticmethod
    # def run():
    #     """ Run the Snips SDK. """
    #     if not Snips.is_installed():
    #         raise SnipsNotFound()

    #     try:
    #         subprocess.Popen(
    #             "snips", stdout=DEVNULL, stderr=subprocess.STDOUT)
    #     except OSError as e:
    #         raise SnipsRuntimeFailure(str(e))

    @staticmethod
    def is_installed():
        """ Check if the Snips SDK is installed. """
        return cmd_exists("snips")

    @staticmethod
    def load_assistant(assistant_zip_path):
        """ Load an assistant file for the Snips SDK.

        :param assistant_zip_path: The path to the assistant.zip file.
        """
        process = subprocess.Popen(
            SNIPS_INSTALL_ASSISTANT_COMMAND.format(assistant_zip_path).split(),
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        output, error = process.communicate()
        return process.returncode



# -*-: coding utf-8 -*-
""" Downloader for Snips assistants. """

import os
import shutil
import subprocess

from .os_helpers import cmd_exists, is_raspi_os, execute_command, pipe_commands

from .. import ASOUNDRC_DEST_PATH, ASOUNDCONF_DEST_PATH

# pylint: disable=too-few-public-methods
class MicrophoneSetup:
    """ Downloader for Snips assistants. """

    ASOUNDRC_CONFIG_PATH = "../config/asoundrc"

    @staticmethod
    def setup_asoundrc(microphone_id):
        if not is_raspi_os():
            return
        if microphone_id == 'respeaker':
            MicrophoneSetup._copy_asoundrc("asoundrc.respeaker")
        elif microphone_id == 'jabra':
            MicrophoneSetup._copy_asoundrc("asoundrc.jabra")
        else:
            MicrophoneSetup._copy_asoundrc("asoundrc.default")

    @staticmethod
    def setup_asoundconf():
        if not is_raspi_os():
            return
        MicrophoneSetup._copy_asoundconf()


    @staticmethod
    def _copy_asoundrc(asoundrc_file):
        """ Copy asoundrc configuration to local path.

        :param asoundrc_file: the name of the asoundrc configuration, as
                              present in the config folder.
        """
        this_dir, this_filename = os.path.split(__file__)
        asoundrc_path = os.path.join(this_dir, MicrophoneSetup.ASOUNDRC_CONFIG_PATH, asoundrc_file)
        destination = os.path.expanduser(ASOUNDRC_DEST_PATH)
        shutil.copy2(asoundrc_path, destination)


    @staticmethod
    def _copy_asoundconf():
        """ Copy asoundconf to local path. """
        this_dir, this_filename = os.path.split(__file__)
        asoundrc_path = os.path.join(this_dir, MicrophoneSetup.ASOUNDRC_CONFIG_PATH, "asound.conf")
        destination = os.path.expanduser(ASOUNDCONF_DEST_PATH)
        shutil.copy2(asoundrc_path, destination)


class RespeakerMicrophoneSetup:

    @staticmethod
    def setup(vendor_id, product_id):
        if not is_raspi_os():
            return

        execute_command("sudo rm -f /lib/udev/rules.d/50-rspk.rules")

        echo_command = ("echo ACTION==\"add\", SUBSYSTEMS==\"usb\", ATTRS{{idVendor}}==\"{}\", " +
                        "ATTRS{{idProduct}}==\"{}\", MODE=\"660\", GROUP=\"plugdev\"") \
            .format(vendor_id, product_id)
        tee_command = "sudo tee --append /lib/udev/rules.d/50-rspk.rules"
        pipe_commands(echo_command, tee_command, silent=True)

        execute_command("sudo adduser pi plugdev")
        execute_command("sudo udevadm control --reload")
        execute_command("sudo udevadm trigger")



# -*-: coding utf-8 -*-
""" Simple caching. """

import os
from .os_helpers import write_text_file, read_file, create_dir, remove_file

from .. import SNIPS_CACHE_DIR

class Cache:

    STORE_FILE = os.path.join(SNIPS_CACHE_DIR, "token_store")

    @staticmethod
    def get_login_token():
        return read_file(Cache.STORE_FILE)

    @staticmethod
    def save_login_token(token):
        write_text_file(Cache.STORE_FILE, token)
    
    @staticmethod
    def clear_login_token():
        remove_file(Cache.STORE_FILE)


# -*-: coding utf-8 -*-
"""The microphone setup command."""

import os
import time

from ...base import Base
from ....utils.os_helpers import is_raspi_os, which
from ....utils.systemd import Systemd

from .... import NODE_MODULES_PATH, DEFAULT_SNIPSFILE_PATH

from snipsskillscore import pretty_printer as pp

class SystemdSnipsSkillsException(Exception):
    pass


class SystemdSnipsSkills(Base):

    SNIPSSKILLS_SERVICE_NAME = "snipsskills"
    SNIPSSKILLS_COMMAND = "snipsskills"

    def run(self):
        snipsfile_path = self.options['--snipsfile_path'] or os.getcwd()
        try:
            SystemdSnipsSkills.setup(snipsfile_path=snipsfile_path)
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def setup(snipsfile_path=None):
        pp.pcommand("Setting up Snips Skills as a Systemd service")

        snipsfile_path = snipsfile_path or DEFAULT_SNIPSFILE_PATH
        working_directory = os.path.dirname(snipsfile_path)
        
        if not is_raspi_os():
            raise SystemdSnipsSkillsException("Snips Systemd configuration is only available on Raspberry Pi. Skipping Systemd setup")

        snipsskills_path = which('snipsskills')
        if snipsskills_path is None:
            raise SystemdSnipsSkillsException("Error: cannot find command 'snipsskills' on the system. Make sure the Snips Skills CLI is correctly installed. Skipping Systemd setup")

        contents = Systemd.get_template(SystemdSnipsSkills.SNIPSSKILLS_SERVICE_NAME)
        contents = contents.replace("{{SNIPSSKILLS_COMMAND}}", snipsskills_path)
        contents = contents.replace("{{WORKING_DIRECTORY}}", working_directory)
        Systemd.write_systemd_file(SystemdSnipsSkills.SNIPSSKILLS_SERVICE_NAME, None, contents)
        Systemd.enable_service(None, SystemdSnipsSkills.SNIPSSKILLS_SERVICE_NAME)

        pp.psuccess("Successfully set up Snips Skills as a Systemd service")



# -*-: coding utf-8 -*-

from ..base import Base
from snipsskillscore import pretty_printer as pp
from ...utils.os_helpers import write_text_file, read_file, file_exists, ask_for_input, ask_for_password
from ...utils.cache import Cache
from ...utils.auth import Auth

# pylint: disable=too-few-public-methods
class Logout(Base):

    def run(self):
        if Cache.get_login_token() is not None:
            Logout.logout()
            pp.psuccess("You are now signed out")
        else:
            pp.psuccess("You are already signed out")

    @staticmethod
    def logout():
        Cache.clear_login_token()



# -*-: coding utf-8 -*-
"""The microphone setup command."""

import time

from ...base import Base
from ....utils.os_helpers import is_raspi_os, which
from ....utils.systemd import Systemd

from .... import NODE_MODULES_PATH

from snipsskillscore import pretty_printer as pp

class SystemdSnipsException(Exception):
    pass


class SystemdSnips(Base):

    SNIPS_SERVICE_NAME = "snips"
    SNIPS_SCRIPT = "{snips_bin_path}"

    def run(self):
        try:
            SystemdSnips.setup()
        except Exception as e:
            pp.perror(str(e))

    @staticmethod
    def setup():
        pp.pcommand("Setting up Snips as a Systemd service")

        if not is_raspi_os():
            raise SystemdSnipsException("Snips Systemd configuration is only available on Raspberry Pi. Skipping Systemd setup")

        snips_path = which('snips')
        if snips_path is None:
            raise SystemdSnipsException("Error: cannot find command 'snips' on the system. Make sure the Snips Platform is correctly installed. Skipping Systemd setup")

        command = SystemdSnips.SNIPS_SCRIPT.format(snips_bin_path=snips_path)
        
        contents = Systemd.get_template(SystemdSnips.SNIPS_SERVICE_NAME)
        contents = contents.replace("{{SNIPS_COMMAND}}", command)
        Systemd.write_systemd_file(SystemdSnips.SNIPS_SERVICE_NAME, None, contents)
        Systemd.enable_service(None, SystemdSnips.SNIPS_SERVICE_NAME)

        pp.psuccess("Successfully set up Snips as a Systemd service")



# -*-: coding utf-8 -*-
""" snipsskills

Usage:
  snipsskills install [--snipsfile=<snipsfile_path> --skip-bluetooth --skip-systemd --force-download --silent] [--email=<email> --password=<password>]
  snipsskills install bluetooth [--force-download]
  snipsskills install skill <skill_url>
  snipsskills install skills [--snipsfile=<snipsfile_path>]
  snipsskills fetch assistant [--snipsfile=<snipsfile_path>] [--id=<id> --url=<url> --file=<file>] [--email=<email> --password=<password>] [--force-download]
  snipsskills load assistant [--file=<file> --platform-only]
  snipsskills setup microphone [--snipsfile=<snipsfile_path>] [<microphone_id> [--skip-asoundrc] [--update-asoundconf] [PARAMS ...]]
  snipsskills setup systemd bluetooth [--mqtt-host=<mqtt_host> --mqtt-port=<mqtt_port>]
  snipsskills setup systemd snips
  snipsskills setup systemd skills [--snipsfile=<snipsfile_path>]
  snipsskills run [--snipsfile=<snipsfile_path>] [--mqtt-host=<mqtt_host> --mqtt-port=<mqtt_port> --tts-service=<tts_service> --locale=<locale>]
  snipsskills login [--email=<email> --password=<password>]
  snipsskills logout
  snipsskills -h | --help
  snipsskills --version

Options:
  -h --help                         Show this screen.
  --version                         Show version.
  --snipsfile=<path>                Path to the Snipsfile.
  --skill=<skillname>

Examples:
  snipsskills install

Help:
  For help using this tool, please open an issue on the Github repository:
  https://github.com/snipsco/snipsskills
"""

import os
import sys

from docopt import docopt
from snipsskillscore import pretty_printer as pp

from . import logger
from . import __version__ as VERSION

def matches_options(options, option_string):
    values = option_string.split("/")
    for value in values:
        if options[value] != True:
            return False
    return True

def main():
    """ Main entry point. """
    options = docopt(__doc__, version=VERSION)

    try:
        if options['setup'] == True and options['microphone'] == True:
            from snipsskills.commands.setup.microphone import MicrophoneInstaller
            MicrophoneInstaller(options).run()
        elif options['setup'] == True and options['systemd'] == True and options['bluetooth'] == True:
            from snipsskills.commands.setup.systemd.bluetooth import SystemdBluetooth
            SystemdBluetooth(options).run()
        elif options['setup'] == True and options['systemd'] == True and options['snips'] == True:
            from snipsskills.commands.setup.systemd.snips import SystemdSnips
            SystemdSnips(options).run()
        elif options['setup'] == True and options['systemd'] == True and options['skills'] == True:
            from snipsskills.commands.setup.systemd.snipsskills import SystemdSnipsSkills
            SystemdSnipsSkills(options).run()
        elif options['login'] == True:
            from snipsskills.commands.session.login import Login
            Login(options).run()
        elif options['logout'] == True:
            from snipsskills.commands.session.logout import Logout
            Logout(options).run()
        elif options['fetch'] == True and options['assistant'] == True:
            from snipsskills.commands.assistant.fetch import AssistantFetcher
            AssistantFetcher(options).run()
        elif options['load'] == True and options['assistant'] == True:
            from snipsskills.commands.assistant.load import AssistantLoader
            AssistantLoader(options).run()
        elif options['install'] == True and options['bluetooth'] == True:
            from snipsskills.commands.install.bluetooth import BluetoothInstaller
            BluetoothInstaller(options).run()
        elif options['install'] == True and options['skill'] == True:
            from snipsskills.commands.install.skill import SkillInstaller
            SkillInstaller(options).run()
        elif options['install'] == True and options['skills'] == True:
            from snipsskills.commands.install.skills import SkillsInstaller
            SkillsInstaller(options).run()
        elif options['install'] == True:
            from snipsskills.commands.install.install import GlobalInstaller
            GlobalInstaller(options).run()
    except KeyboardInterrupt:
        try:
            print("\n")
            pp.perror("Snips Skills installer interrupted")
            sys.exit(0)
        except SystemExit:
            os._exit(0)

    try:
        if options['run'] == True:
            from snipsskills.commands.run import Runner
            Runner(options).run()
    except KeyboardInterrupt:
        try:
            print("\n")
            logger.error("Snips Skills server stopped")
            sys.exit(0)
        except SystemExit:
            os._exit(0)

    # elif options['scaffold'] == True:
    #     from snipsskills.commands.scaffold import Scaffold
    #     Scaffold().run()

if __name__ == '__main__':
    main()


# -*-: coding utf-8 -*-

import hashlib
import os
import shutil
import time

from ..base import Base
from ..session.login import Login
from ..session.logout import Logout
from ...utils.http_helpers import fetch_url
from ...utils.os_helpers import write_binary_file, file_exists
from ...utils.cache import Cache
from ...utils.snipsfile import Snipsfile

from ... import SNIPS_CACHE_DIR, DEFAULT_SNIPSFILE_PATH

from snipsskillscore import pretty_printer as pp

class AssistantFetcherException(Exception):
    pass

# pylint: disable=too-few-public-methods
class AssistantFetcher(Base):

    SNIPS_TEMP_ASSISTANT_FILENAME = "assistant.zip"
    SNIPS_TEMP_ASSISTANT_PATH = os.path.join(SNIPS_CACHE_DIR, SNIPS_TEMP_ASSISTANT_FILENAME)
    CONSOLE_ASSISTANT_URL = "https://external-gateway.snips.ai/v1/assistant/{}/download"

    def run(self):
        """ Command runner.

        Docopt command:
        
        snipsskills fetch assistant [--id=<id> --url=<url> --file=<filename>]
        """
        force_download = self.options['--force-download']
        try:
            aid = self.options['--id']
            url = self.options['--url']
            file = self.options['--file']
            email = self.options['--email']
            password = self.options['--password']
            if aid is not None or url is not None or file is not None:
                AssistantFetcher.fetch_from_params(aid=aid, url=url, file=file, email=email, password=password, force_download=force_download)
            else:
                AssistantFetcher.fetch(self.options['--snipsfile'], email=email, password=password, force_download=force_download)
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def fetch(snipsfile_path=None, email=None, password=None, force_download=False):
        if snipsfile_path is None:
            snipsfile_path = DEFAULT_SNIPSFILE_PATH
        if snipsfile_path is not None and not file_exists(snipsfile_path):
            raise AssistantFetcherException("Error fetching assistant: Snipsfile not found")
        snipsfile = Snipsfile(snipsfile_path)
        AssistantFetcher.fetch_from_snipsfile(snipsfile, email=email, password=password, force_download=force_download)


    @staticmethod
    def fetch_from_snipsfile(snipsfile, email=None, password=None, force_download=False):
        if snipsfile is None:
            raise AssistantFetcherException("Error fetching assistant: Snipsfile not found")
        AssistantFetcher.fetch_from_params(aid=snipsfile.assistant_id, url=snipsfile.assistant_url, file=snipsfile.assistant_file, email=email, password=password, force_download=force_download)


    @staticmethod
    def fetch_from_params(aid=None, url=None, file=None, email=None, password=None, force_download=False):
        pp.pcommand("Fetching assistant")

        if aid is None and url is None and file is None:
            raise AssistantFetcherException("Error fetching assistant. Please provide an assistant ID, a public URL, or a filename")

        if url:
            AssistantFetcher.download_public_assistant(url, force_download=force_download)
        elif aid:
            AssistantFetcher.download_console_assistant(aid, email=email, password=password, force_download=force_download)
        elif file:
            AssistantFetcher.copy_local_file(file)


    @staticmethod
    def download_public_assistant(url, force_download=False):
        message = pp.ConsoleMessage("Downloading assistant from {}".format(url))
        message.start()

        if not force_download and AssistantFetcher.exists_cached_from_url(url):
            message.done()
            pp.psubsuccess("Using cached version from {}".format(SNIPS_CACHE_DIR))
            AssistantFetcher.copy_to_temp_assistant_from_url(url)
            return

        try:
            content = fetch_url(url)
            message.done()
        except:
            raise AssistantFetcherException("Error downloading assistant. Please make sure the assistant URL is correct")

        filepath = AssistantFetcher.get_assistant_cache_path_from_url(url)
        message = pp.ConsoleMessage("Saving assistant to {}".format(SNIPS_CACHE_DIR))
        message.start()
        write_binary_file(filepath, content)
        message.done()
        AssistantFetcher.copy_to_temp_assistant_from_url(url)


    @staticmethod
    def download_console_assistant(aid, email=None, password=None, force_download=False):
        start_message = "Fetching assistant $GREEN{}$RESET from the Snips Console".format(aid)

        if not force_download and AssistantFetcher.exists_cached_from_assistant_id(aid):
            pp.psubsuccess(start_message)
            pp.psubsuccess("Using cached version: {}".format(AssistantFetcher.get_assistant_cache_path_from_assistant_id(aid)))
            AssistantFetcher.copy_to_temp_assistant_from_assistant_id(aid)
            return

        if email is not None and password is not None:
            Logout.logout()

        token = Cache.get_login_token()
        if token is None:
            token = AssistantFetcher.get_token(email=email, password=password)

        message = pp.ConsoleMessage(start_message)
        message.start()
        try:
            content = AssistantFetcher.download_console_assistant_only(aid, token)
            message.done()
        except Exception as e:
            message.error()
            Logout.logout()
            token = AssistantFetcher.get_token(email=email, password=password)
            message = pp.ConsoleMessage("Retrying to fetch assistant $GREEN{}$RESET from the Snips Console".format(aid))
            message.start()
            try:
                content = AssistantFetcher.download_console_assistant_only(aid, token)
                message.done()
            except Exception:
                message.error()
                raise AssistantFetcherException("Error fetching assistant from the console. Please make sure the ID is correct, and that you are signed in")

        filepath = AssistantFetcher.get_assistant_cache_path_from_assistant_id(aid)
        message = pp.ConsoleMessage("Saving assistant to {}".format(filepath))
        message.start()
        write_binary_file(filepath, content)
        message.done()
        AssistantFetcher.copy_to_temp_assistant_from_assistant_id(aid)


    @staticmethod
    def download_console_assistant_only(aid, token):
        url = AssistantFetcher.CONSOLE_ASSISTANT_URL.format(aid)
        return fetch_url(url, headers={'Authorization': token, 'Accept': 'application/json'})


    @staticmethod
    def exists_cached_from_url(url):
        return file_exists(AssistantFetcher.get_assistant_cache_path_from_url(url))


    @staticmethod
    def exists_cached_from_assistant_id(assistant_id):
        return file_exists(AssistantFetcher.get_assistant_cache_path_from_assistant_id(assistant_id))


    @staticmethod
    def exists_assistant_filename(filename):
        return file_exists(AssistantFetcher.get_assistant_file_path(filename))


    @staticmethod
    def get_assistant_filename_from_url(url):
        return "assistant_" + hashlib.sha224(url).hexdigest() + ".zip"


    @staticmethod
    def get_assistant_filename_from_assistant_id(assistant_id):
        return "assistant_{}.zip".format(assistant_id)


    @staticmethod
    def get_assistant_cache_path_from_url(url):
        return AssistantFetcher.get_assistant_file_path(AssistantFetcher.get_assistant_filename_from_url(url))


    @staticmethod
    def get_assistant_cache_path_from_assistant_id(assistant_id):
        return AssistantFetcher.get_assistant_file_path(AssistantFetcher.get_assistant_filename_from_assistant_id(assistant_id))


    @staticmethod
    def get_assistant_file_path(filename):
        return os.path.join(SNIPS_CACHE_DIR, filename)


    @staticmethod
    def get_token(email=None, password=None):
        try:
            return Login.login(email=email, password=password, greeting="Please enter your Snips Console credentials to download your assistant.", silent=True)
        except Exception as e:
            raise AssistantFetcherException("Error logging in: {}".format(str(e)))


    @staticmethod
    def copy_to_temp_assistant_from_url(url):
        AssistantFetcher.copy_local_file(AssistantFetcher.get_assistant_cache_path_from_url(url), silent=True)
    

    @staticmethod
    def copy_to_temp_assistant_from_assistant_id(assistant_id):
        AssistantFetcher.copy_local_file(AssistantFetcher.get_assistant_cache_path_from_assistant_id(assistant_id), silent=True)


    @staticmethod
    def copy_local_file(file_path, silent=False):
        if not silent:
            message = pp.ConsoleMessage("Copying assistant {} to {}".format(file_path, AssistantFetcher.SNIPS_TEMP_ASSISTANT_PATH))
            message.start()
        
        error = None
        if not file_exists(file_path):
            error = "Error: failed to locate file {}".format(file_path)
        else:
            try:
                shutil.copy2(file_path, AssistantFetcher.SNIPS_TEMP_ASSISTANT_PATH)
            except Exception as e:
                error = "Error: failed to copy file {}. Make sure you have write permissions to {}".format(file_path, AssistantFetcher.SNIPS_TEMP_ASSISTANT_PATH)

        if error is not None:
            if not silent:
                message.error()
            raise AssistantFetcherException(error)
        else:
            if not silent:
                message.done()



# -*-: coding utf-8 -*-

import os

from ..base import Base
from ...utils.pip_installer import PipInstaller

from snipsskillscore import pretty_printer as pp

class SkillInstallerException(Exception):
    pass

class SkillInstallerWarning(Exception):
    pass


class SkillInstaller(Base):
    
    def run(self, force_download=False):
        url_or_pip = self.options['<skill_url>']

        try:
            SkillInstaller.install(url_or_pip, force_download=force_download)
        except SkillInstallerWarning as e:
            pp.pwarning(str(e))
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def install(url_or_pip, force_download=False):
        message = pp.ConsoleMessage("Installing skill: $GREEN{}$RESET".format(url_or_pip))
        message.start()
        # try:
        PipInstaller.install(url_or_pip, force_download=force_download)
        message.done()
        # except Exception as e:
        #     message.error()
        #     raise SkillInstallerWarning("Error installing skill {}: make sure you have the required access rights, and that the module is available".format(url_or_pip))



# -*-: coding utf-8 -*-
""" pip module installer. """

import os
import pip

from .os_helpers import execute_command, is_valid_github_url, read_file

from .. import SNIPS_CACHE_DIR
from .. import DEB_VENV, SHELL_COMMAND, PIP_BINARY

class PipInstallerException(Exception):
    pass

# pylint: disable=too-few-public-methods
class PipInstaller:
    """ pip module installer. """

    @staticmethod
    def install(url_or_pip, force_download=False):
        """ Install a Python module.

        :param url_or_pip: URL of the module, or pip ID.
        """
        if is_valid_github_url(url_or_pip):
            PipInstaller.install_url(url_or_pip, force_download=force_download)
        else:
            PipInstaller.install_pip(url_or_pip, force_download=force_download)


    @staticmethod
    def install_pip(package_name, force_download=False):
        no_cache = "--no-cache" if force_download else ""
        PipInstaller.execute_pip_install("--upgrade --quiet {} {}".format(no_cache, package_name))
    

    @staticmethod
    def install_url(url, force_download=False):
        if url.startswith("https://"):
            url = "git+" + url

        if not force_download and PipCache.is_installed(url):
            return

        PipInstaller.execute_pip_install("--upgrade --quiet {}".format(url))
        PipCache.add(url)


    @staticmethod
    def execute_pip_install(arguments):
        is_venv_active = PipInstaller.activate_venv()
        command = "{} {}".format(PIP_BINARY, arguments)
        (output, error) = execute_command("{} install {}".format(PIP_BINARY, arguments), silent=False)
        if is_venv_active:
            PipInstaller.deactivate_venv()
        if error is not None and error.strip() != '':
            raise PipInstallerException(error)

    @staticmethod
    def activate_venv():
        try:
            execute_command("{} {}/bin/activate".format(SHELL_COMMAND, DEB_VENV), silent=True)
            return True
        except Exception as e:
            return False

    @staticmethod
    def deactivate_venv():
        try:
            execute_command("deactivate", silent=True)
        except:
            pass


class PipCache:

    STORE_FILE = os.path.join(SNIPS_CACHE_DIR, "pip_cache")

    @staticmethod
    def is_installed(github_url):
        cache = read_file(PipCache.STORE_FILE)
        if cache is not None and cache.strip() != '':
            return cache.find(github_url) != -1
        return False

    @staticmethod
    def add(github_url):
        if PipCache.is_installed(github_url):
            return
        with open(PipCache.STORE_FILE, "a") as f:
            f.write(github_url + "\n")



# -*-: coding utf-8 -*-
"""The run command."""
# pylint: disable=too-few-public-methods,import-error,wrong-import-position,wrong-import-order,wildcard-import,wrong-import-position,wrong-import-order

import os
import subprocess
import time
import threading

from sys import path

from .base import Base
from ..utils.snipsfile import Snipsfile
from ..utils.os_helpers import file_exists

from .. import DEFAULT_SNIPSFILE_PATH, SNIPS_CACHE_INTENTS_DIR, logger

from snipsskillscore.server import Server
# This is used potentially by code blocks in Snipsfile and Snipsspec files.
from snipsskillscore.instant_time import InstantTime
from snipsskillscore.time_interval import TimeInterval


path.append(SNIPS_CACHE_INTENTS_DIR)
path.append(os.path.join(SNIPS_CACHE_INTENTS_DIR, "intents"))

from intent_registry import IntentRegistry
from intents import *

#INTENT_REGISTRY_FILE = ".snips/intents/intent_registry.py"


class RunnerException(Exception):
    pass


class Runner(Base):
    """The run command."""

    # pylint: disable=undefined-variable,exec-used,eval-used
    def run(self):
        """ Command runner. """
        try:
            snipsfile = self.options['--snipsfile']
            mqtt_hostname = self.options['--mqtt-host']
            mqtt_port = self.options['--mqtt-port']
            tts_service_id = self.options['--tts-service']
            locale = self.options['--locale']
            Runner.run_from_snipsfile_path(snipsfile_path=snipsfile, mqtt_hostname=mqtt_hostname, mqtt_port=mqtt_port, tts_service_id=tts_service_id, locale=locale)

        except Exception as e:
            logger.error(str(e))


    @staticmethod
    def run_from_snipsfile_path(snipsfile_path=None, mqtt_hostname=None, mqtt_port=None, tts_service_id=None, locale=None):
        snipsfile_path = snipsfile_path or DEFAULT_SNIPSFILE_PATH
        if snipsfile_path is not None and not file_exists(snipsfile_path):
            raise RunnerException("Error running skills server: Snipsfile not found")
        snipsfile = Snipsfile(snipsfile_path)
        Runner.run_from_snipsfile(snipsfile, mqtt_hostname=mqtt_hostname, mqtt_port=mqtt_port, tts_service_id=tts_service_id, locale=locale)


    @staticmethod
    def run_from_snipsfile(snipsfile, mqtt_hostname=None, mqtt_port=None, tts_service_id=None, locale=None):
        Runner.run_with_params(
            mqtt_hostname=mqtt_hostname or snipsfile.mqtt_hostname,
            mqtt_port=mqtt_port or snipsfile.mqtt_port,
            tts_service_id=tts_service_id or snipsfile.tts_service,
            locale=locale or snipsfile.locale,
            skilldefs=snipsfile.skilldefs)


    @staticmethod
    def run_with_params(mqtt_hostname, mqtt_port, tts_service_id, locale, skilldefs=[]):
        skillsRunner = SkillsRunner(mqtt_hostname, mqtt_port, tts_service_id, locale, skilldefs)
        skillsRunner.start()


class SkillsRunner:

    def __init__(self, mqtt_hostname, mqtt_port, tts_service_id, locale, skilldefs=[]):
        logger.info("Starting Snips Skills")

        self.registry = IntentRegistry()
        self.server = Server(mqtt_hostname, mqtt_port, tts_service_id, locale, self.registry, self.handle_intent_async, logger)

        self.skilldefs = skilldefs
        self.skills = {}
        for skilldef in self.skilldefs:
            try:
                logger.info("Loading skill {}".format(skilldef.package_name))
                if skilldef.package_name is None:
                    logger.error("Error loading skill: required entry 'package_name' not found for the skill definition in your Snipsfile")
                    continue
                class_name = skilldef.class_name or "Skill"
                module_name = skilldef.package_name + "." + skilldef.package_name
                exec("from {} import {}".format(module_name, class_name))
                cls = eval(class_name)
                skill_instance = cls(**skilldef.params)
                if skilldef.requires_tts:
                    skill_instance = cls(tts_service=self.server.tts_service, **skilldef.params)
                else:
                    skill_instance = cls(**skilldef.params)
                self.skills[skilldef.package_name] = skill_instance
                logger.info("Successfully loaded skill {}".format(skilldef.package_name))
            except Exception as e:
                logger.error("Error loading skill {}: {}".format(
                    skilldef.package_name, str(e)))

    def start(self):
        logger.info("Starting the Snips Skills server.")
        self.server.start()

    def handle_intent_async(self, intent, payload=None):
        """ Handle an intent asynchronously.

        :param intent: the incoming intent to handle.
        """
        thread = threading.Thread(target=self.handle_intent, args=(intent, payload, ))
        thread.start()

    def handle_intent(self, intent, payload=None):
        """ Handle an intent.

        :param intent: the incoming intent to handle.
        """
        for skilldef in self.skilldefs:
            intent_def = skilldef.find(intent)
            if intent_def is None:
                continue
            if not skilldef.package_name in self.skills:
                continue
            skill = self.skills[skilldef.package_name]
            if intent_def.action.startswith("{%"):
                # Replace variables in scope with random variables
                # to prevent the skill from accessing/editing them.
                action = intent_def.action \
                    .replace("{%", "") \
                    .replace("%}", "") \
                    .replace("skilldef", "_snips_eejycfyrdfzilgfb") \
                    .replace("intent_def", "_snips_jkqdruouzuahmgns") \
                    .replace("snipsfile", "_snips_pdzdcpaygyjklngz") \
                    .replace("tts_service", "_snips_bxzbomfguxlyxswo") \
                    .strip()
                exec(action)
            else:
                getattr(skill, intent_def.action)()



# -*-: coding utf-8 -*-
"""The microphone setup command."""

import time

from ..base import Base
from ...utils.os_helpers import file_exists
from ...utils.os_helpers import is_raspi_os
from ...utils.microphone_setup import MicrophoneSetup, RespeakerMicrophoneSetup
from ...utils.snipsfile import Snipsfile

from ... import DEFAULT_SNIPSFILE_PATH, ASOUNDRC_DEST_PATH

from snipsskillscore import pretty_printer as pp

class MicrophoneInstallerException(Exception):
    pass

class MicrophoneInstallerWarning(Exception):
    pass


# pylint: disable=too-few-public-methods
class MicrophoneInstaller(Base):
    """The microphone setup command."""

    def run(self):
        """ Command runner.

        Docopt command:
        
        snipsskills setup microphone [--snipsfile=<snipsfile>] [<microphone_id> [--skip-asoundrc] [--update_asoundconf] [PARAMS ...]]
        """
        try:
            microphone_id = self.options['<microphone_id>']
            if microphone_id is not None:
                update_asoundrc = not self.options['--skip-asoundrc']
                update_asoundconf = self.options['--update-asoundconf']
                params = self.options['PARAMS']
                MicrophoneInstaller.install_from_params(microphone_id, update_asoundrc, update_asoundconf, params_list=params)
            else:
                MicrophoneInstaller.install(snipsfile_path=self.options['--snipsfile'])
        except MicrophoneInstallerWarning as e:
            pp.pwarning(str(e))
        except Exception as e:
            pp.perror(str(e))

    @staticmethod
    def install(snipsfile_path=None, silent=False):
        MicrophoneInstaller.print_start(silent=silent)
        if snipsfile_path is None:
            snipsfile_path = DEFAULT_SNIPSFILE_PATH
        if snipsfile_path is not None and not file_exists(snipsfile_path):
            raise SkillsInstallerException("Error setting up microphone: Snipsfile not found")
        snipsfile = Snipsfile(snipsfile_path)
        MicrophoneInstaller.install_from_snipsfile(snipsfile, silent=True)
        MicrophoneInstaller.print_done(silent)


    @staticmethod
    def install_from_snipsfile(snipsfile, silent=False):
        MicrophoneInstaller.print_start(silent=silent)
        if snipsfile is None:
            raise MicrophoneInstallerException("Error setting up microphone: Snipsfile not found")

        microphone_id = snipsfile.microphone_config.identifier
        params_dict = snipsfile.microphone_config.params
        modify_asoundrc = snipsfile.modify_asoundrc
        modify_asoundconf = snipsfile.modify_asoundconf

        MicrophoneInstaller.install_from_params(microphone_id, modify_asoundrc, modify_asoundconf, params_dict=params_dict, silent=True)
        MicrophoneInstaller.print_done(silent)


    @staticmethod
    def install_from_params(microphone_id, update_asoundrc, update_asoundconf, params_list=None, params_dict=None, silent=False):
        MicrophoneInstaller.print_start(microphone_id, silent)

        if not is_raspi_os():
            raise MicrophoneInstallerWarning("System is not Raspberry Pi. Skipping microphone setup")
        
        if update_asoundrc:
            message = pp.ConsoleMessage("Copying .asoundrc to {}".format(ASOUNDRC_DEST_PATH))
            message.start()
            MicrophoneSetup.setup_asoundrc(microphone_id)
            message.done()
            
        if update_asoundconf:
            message = pp.ConsoleMessage("Copying asound.conf to {}".format(MicrophoneSetup.ASOUNDCONF_DEST_PATH))
            message.start()
            MicrophoneSetup.setup_asoundconf()
            message.done()

        if microphone_id == 'respeaker':
            message = pp.ConsoleMessage("Installing ReSpeaker drivers")
            message.start()
            try:
                respeaker_params = MicrophoneInstaller.normalize_respeaker_params(params_list=params_list, params_dict=params_dict)
                RespeakerMicrophoneSetup.setup(respeaker_params['vendorId'], respeaker_params['productId'])
                message.done()
            except MicrophoneInstallerException as e:
                message.error()
                raise MicrophoneInstallerException(str(e))

        MicrophoneInstaller.print_done(silent)


    @staticmethod
    def normalize_respeaker_params(params_list=None, params_dict=None):
        vendorId = None
        productId = None
        exception = MicrophoneInstallerException("Error installing ReSpeaker drivers: you must provide both vendor id and product id")
        if params_list is not None:
            if len(params_list) < 2:
                raise exception
            vendorId = params_list[0]
            productId = params_list[1]
        elif params_dict is not None:
            if not 'vendorId' in params_dict and not 'productId' in params_dict:
                raise exception
            vendorId = params_dict['vendorId']
            productId = params_dict['productId']
        return { 'vendorId': vendorId, 'productId': productId }


    @staticmethod
    def print_start(microphone_id=None, silent=False):
        if not silent:
            if microphone_id is not None:
                pp.pcommand("Setting up microphone: {}".format(microphone_id))
            else:
                pp.pcommand("Setting up microphone")


    @staticmethod
    def print_done(silent=False):
        if not silent:
            pp.psuccess("Microphone setup successfully complete")



# -*-: coding utf-8 -*-

import os

from ..base import Base
from ...utils.os_helpers import file_exists
from ...utils.os_helpers import is_raspi_os
from ...utils.snipsfile import Snipsfile

from ..assistant.fetch import AssistantFetcher
from ..assistant.load import AssistantLoader
from ..setup.microphone import MicrophoneInstaller
from ..setup.systemd.bluetooth import SystemdBluetooth
from ..setup.systemd.snips import SystemdSnips
from ..setup.systemd.snipsskills import SystemdSnipsSkills
from .skills import SkillsInstaller, SkillsInstallerWarning
from .bluetooth import BluetoothInstaller

from ... import DEFAULT_SNIPSFILE_PATH

from snipsskillscore import pretty_printer as pp

class GlobalInstallerException(Exception):
    pass

class GlobalInstallerWarning(Exception):
    pass


class GlobalInstaller(Base):
    
    def run(self):
        pp.silent = self.options['--silent']
        try:
            GlobalInstaller.install(self.options['--snipsfile'], skip_bluetooth=self.options['--skip-bluetooth'], skip_systemd=self.options['--skip-systemd'], email=self.options['--email'], password=self.options['--password'], force_download=self.options['--force-download'])
        except GlobalInstallerWarning as e:
            pp.pwarning(str(e))
        except Exception as e:
            pp.perror(str(e))


    @staticmethod
    def install(snipsfile_path=None, skip_bluetooth=False, skip_systemd=False, email=None, password=None, force_download=False):
        snipsfile_path = snipsfile_path or DEFAULT_SNIPSFILE_PATH
        if snipsfile_path is not None and not file_exists(snipsfile_path):
            raise GlobalInstallerException("Error running installer: Snipsfile not found")
        snipsfile = Snipsfile(snipsfile_path)
        GlobalInstaller.install_from_snipsfile(snipsfile, skip_bluetooth=skip_bluetooth, skip_systemd=skip_systemd, email=email, password=password, force_download=force_download)


    @staticmethod
    def install_from_snipsfile(snipsfile, skip_bluetooth=False, skip_systemd=False, email=None, password=None, force_download=False):
        pp.pheader("Running Snips Skills installer")

        if snipsfile is None:
            raise GlobalInstallerException("Error running installer: no Snipsfile provided")

        try:
            AssistantFetcher.fetch(email=email, password=password, force_download=force_download)
            AssistantLoader.load()
        except Exception as e:
            pp.pwarning(str(e))

        try:
            SkillsInstaller.install(force_download=force_download)
        except Exception as e:
            pp.pwarning(str(e))
        
        try:
            MicrophoneInstaller.install()
        except Exception as e:
            pp.pwarning(str(e))

        if not skip_bluetooth and is_raspi_os():
            try:
                BluetoothInstaller.install(force_download=force_download)
            except Exception as e:
                pp.pwarning(str(e))

        if not skip_systemd and is_raspi_os():
            try:
                SystemdBluetooth.setup()
            except Exception as e:
                pp.pwarning(str(e))
            try:
                SystemdSnips.setup()
            except Exception as e:
                pp.pwarning(str(e))
            try:
                SystemdSnipsSkills.setup()
            except Exception as e:
                pp.pwarning(str(e))

        if not skip_systemd:
            pp.pheadersuccess("Snips Skills installer complete! You can now reboot your device, or manually run 'snipsskills run' to start the Snips Skills server")
        else:
            pp.pheadersuccess("Snips Skills installer complete! Now run 'snipsskills run' to start the Snips Skills server.")
